﻿//====================================
//  HETN-DIO864T/HWIF-DIO864W
//  Library function source for Windows
//	   File name  : ed864l1a.c
//     Last modify: 2019/06/11
//	Copyright(C) 2018-2019  Hivertec Inc.  
//  All Rights Reserved.
//====================================

#include	<windows.h>
#include	<stdio.h>
#include	"ed864l1a.h"
#include	"hedio864.h"

//=============================================================================
//							常駐プログラム
//=============================================================================

// 常駐プログラムリンク設定
// 引数：リンクIPモード, ＩＰアドレス（リンクモード＝1の時有効）, ポート番号（リンクモード＝1の時または2で0以外の時有効）
DWORD hed864_ProgLinkSet(short sMode, BYTE* sIp, WORD usPort)
{
	DWORD	dwRet;			// 戻り値
	BYTE	byInf[64];		// for 8 info.(8×8)
	BYTE	byTmp[16];
	BYTE	byAdr[16];
	BYTE	byIp[4];
	WORD	usTprt;
	WORD	usMax;
	short	i;
	char*	tok;
	char*	ctx;

	if (sMode == 0) {
		// 常駐ﾌﾟﾛｸﾞﾗﾑ自動検索したPCへのﾘﾝｸ
		// ｱﾄﾞﾚｽとﾎﾟｰﾄの自動検索
		usMax = 8;	/* 最大バッファ数 8情報分 */
		dwRet = ed864_EtnSearch(byInf, &usMax, 2000);     /* 2000ms waitting time/max 4 informations) */
		if(dwRet != 0) return dwRet;
		/* sIpで有効なネットワークアドレスが指定されていない場合は先頭情報を使用（デフォルト） */
		sprintf_s(byAdr, sizeof(byAdr), "%d.%d.%d.%d", byInf[0], byInf[1], byInf[2], byInf[3] );
		usTprt = (WORD)byInf[7] * 0x100 + (WORD)byInf[6];

		/* 複数情報がある場合でかつネットワークアドレス指定がある場合は検索 */
		if(usMax >= 2) {
			byIp[0] = 0;
			byIp[1] = 0;
			byIp[2] = 0;
			byIp[3] = 0;
			i = 0;
			if((sIp != "") && (sIp != NULL)) {
				memcpy(byTmp, sIp, sizeof(byTmp));
				tok = strtok_s(byTmp, ".", &ctx);
				while(tok != NULL) {
					byIp[i] = (BYTE)atoi(tok);
					i++;
					if(i >= 3) break;
					tok = strtok_s(NULL, ".", &ctx);
				}
			}
			if(i >= 3) {
				for(i=0; i<usMax; i++) {
					if((byInf[i*8+0] == byIp[0]) && (byInf[i*8+1] == byIp[1]) && (byInf[i*8+2] == byIp[2])) {
						sprintf_s(byAdr, sizeof(byAdr), "%d.%d.%d.%d", byInf[i*8+0], byInf[i*8+1], byInf[i*8+2], byInf[i*8+3] );
						usTprt = (WORD)byInf[i*8+7] * 0x100 + (WORD)byInf[i*8+6];
						break;
					}
				}
			}
		}
		dwRet = ed864_SetProgLink(byAdr, usTprt, RESP_TMO);    /* Local network address */
		if (dwRet != 0) return dwRet;
	}
	else if (sMode == 1) {
		// 指定PC常駐ﾌﾟﾛｸﾞﾗﾑへのﾘﾝｸ
		dwRet = ed864_SetProgLink(sIp, usPort, RESP_TMO);
		if (dwRet != 0) return dwRet;
	}
	else if (sMode == 2) {
		// 自PCのﾛｰｶﾙﾙｰﾌﾟﾊﾞｯｸｱﾄﾞﾚｽでのﾘﾝｸ
		usMax = 8;	/* 最大バッファ数 8情報分 */
		dwRet = ed864_EtnSearch(byInf, &usMax, 2000);     /* 2000ms waitting time/max 4 informations) */
		if ((dwRet != 0) && (dwRet != NETDEV_NOTFOUND)) return dwRet;
		if(usPort != 0) {
			usTprt = usPort;
		}
		// ﾎﾟｰﾄは同じはずなので先頭情報を使用
		else if(dwRet == 0) {
			usTprt = (WORD)byInf[7] * 0x100 + (WORD)byInf[6];
		}
		else {
			usTprt = 10010;
		}
		dwRet = ed864_SetProgLink("127.0.0.1", usTprt, RESP_TMO);    /* Local loopback address */
		if (dwRet != 0) return dwRet;
	}
	else {
		return ILLEGAL_NETPARAM;
	}
	return dwRet;
}


// 常駐プログラムソフトウェアバージョン読出
// 引数：常駐プログラムバージョン格納アドレス，アクセスライブラリバージョン格納アドレス
DWORD hed864_GetProgVersion(WORD *wPver, WORD *wLver)
{
	DWORD	dwRet;			// 戻り値

	// プログラムバージョン読出
	dwRet = ed864_GetProgVersion(wPver, wLver);
	return dwRet;
}

//=============================================================================
//							動作情報取得
//=============================================================================
// コマンド実行エラーの取得 2017.3.6/2017.3.28ﾊﾞｸﾞ修正
// 引数：デバイスタイプ，ボードＩＤ，コマンド実行時発生したエラー
DWORD hed864_GetCmdError(short wType, short wBid, DWORD *err)
{
	BYTE by0xE1Cmd[10] = { 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
						0xE1, 0x00,  };
	BYTE by0xE0Cmd[10] = { 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
						0xE0, 0x01 };
	union {
		DWORD	d[6];
		WORD	w[12];
		BYTE	b[24];
	} byRes;
	WORD	wCsz;
	WORD	wRsz;
	DWORD	dwRet;			// 戻り値

	by0xE1Cmd[0] = (BYTE)wBid;	// set BID
	wCsz = sizeof(by0xE1Cmd);	// set command data size
	wRsz = sizeof(byRes);		// set response data size
	dwRet = ed864_EtnSend(wType, wBid, by0xE1Cmd, &wCsz);
	if(dwRet) return dwRet;
	dwRet = ed864_EtnRecv(wType, wBid, &byRes.b[0], &wRsz);
	if(dwRet) return dwRet;
	*err = byRes.d[2];	// エラー情報

	if(*err != 0) {		// エラーがある場合はクリアコマンド実行
		by0xE0Cmd[0] = (BYTE)wBid;	// set BID
		wCsz = sizeof(by0xE0Cmd);	// set command data size
		wRsz = 8;					// set response data size(header only)
		dwRet = ed864_EtnSend(wType, wBid, by0xE0Cmd, &wCsz);
		if(dwRet) return dwRet;
		dwRet = ed864_EtnRecv(wType, wBid, &byRes.b[0], &wRsz);
	}
	return dwRet;
}

