﻿
using System;
using System.Runtime.InteropServices;
using System.Threading;

/// <summary>
/// HETN-DIO864T/HWIF-DIO864W
/// Library lebel1a function for Visual C#
///     File name	: Ed864l1a.cs
///     Class name	: Ed864l1a
///     Last modify	: 2019/06/11
/// Copyright(C) 2018-2019  Hivertec,inc. All Rights Reserved.
/// </summary>
public class Ed864l1a
{

    /// Ethernet/WiFi受信タイムアウト時間（3000ms）
    /// WiFiを使用している場合は通信中に回線が切断される場合
    /// がありますのでタイムアウト発生時のリトライ処理を
    /// 組み込むことを推奨します
    public const ushort RESP_TMO = 3000;


    //=============================================================================
    //							常駐プログラム
    //=============================================================================

    /// <summary>
    /// 常駐プログラムリンク設定
    /// </summary>
    /// <param name="sMode">リンクIPモード</param>
    /// <param name="sIp">ＩＰアドレス（リンクモード＝1の時有効）</param>
    /// <param name="usPort">ポート番号（リンクモード＝1の時または2で0以外の時有効）</param>
    /// <returns>0:正常，0以外:異常</returns>
    public static uint hed864_ProgLinkSet(short sMode, string sIp, ushort usPort)
    {
        uint unRet = 0;
        byte[] byInf = new byte[64];
        byte[] byAdr = new byte[16];
        byte[] byIp = new byte[14];
        short i;
        ushort usTprt;
        ushort usMax;
        string cBuf;
        string[] cRes;

        if (sMode == 0)
        {
            // 常駐ﾌﾟﾛｸﾞﾗﾑ自動検索したPCへのﾘﾝｸ
            // ｱﾄﾞﾚｽとﾎﾟｰﾄの自動検索
            usMax = 8;	/* 最大バッファ数 8情報分 */
            unRet = Hedio864.ed864_EtnSearch(byInf, ref usMax, 2000);     /* 2000ms waitting time/max 4 informations) */
            if (unRet != 0) return unRet;
            /* sIpで有効なネットワークアドレスが指定されていない場合は先頭情報を使用（デフォルト） */
            cBuf = byInf[0].ToString("D") + "." + byInf[1].ToString("D") + "." + byInf[2].ToString("D") + "." + byInf[3].ToString("D");
            byAdr = System.Text.Encoding.ASCII.GetBytes(cBuf);
            usTprt = (ushort)((ushort)byInf[7] * 0x100 + (ushort)byInf[6]);

            /* 複数情報がある場合でかつネットワークアドレス指定がある場合は検索 */
            if (usMax >= 2)
            {
                byIp[0] = 0;
                byIp[1] = 0;
                byIp[2] = 0;
                byIp[3] = 0;
                i = 0;
                if ((sIp != "") && (sIp != null))
                {
                    cRes = sIp.Split('.');
                    foreach (string cTmp in cRes)
                    {
                        Byte.TryParse(cTmp, out byIp[i]);
                        i += 1;
                    }
                }
                if (i >= 3)
                {
                    for (i = 0; i < usMax; i++)
                    {
                        if ((byInf[i * 8 + 0] == byIp[0]) && (byInf[i * 8 + 1] == byIp[1]) && (byInf[i * 8 + 2] == byIp[2]))
                        {
                            cBuf = byInf[i * 8 + 0].ToString("D") + "." + byInf[i * 8 + 1].ToString("D") + "." + byInf[i * 8 + 2].ToString("D") + "." + byInf[i * 8 + 3].ToString("D");
                            byAdr = System.Text.Encoding.ASCII.GetBytes(cBuf);
                            usTprt = (ushort)((ushort)byInf[i * 8 + 7] * 0x100 + (ushort)byInf[i * 8 + 6]);
                            break;
                        }
                    }
                }
            }
            unRet = Hedio864.ed864_SetProgLink(byAdr, usTprt, RESP_TMO);    /* Local network address */
            if (unRet != 0) return unRet;
        }
        else if (sMode == 1)
        {
            // 指定PC常駐ﾌﾟﾛｸﾞﾗﾑへのﾘﾝｸ
            byAdr = System.Text.Encoding.ASCII.GetBytes(sIp);
            unRet = Hedio864.ed864_SetProgLink(byAdr, usPort, RESP_TMO);
            if (unRet != 0) return unRet;
        }
        else if (sMode == 2)
        {
            // 自PCのﾛｰｶﾙﾙｰﾌﾟﾊﾞｯｸｱﾄﾞﾚｽでのﾘﾝｸ
            usMax = 8;	/* 最大バッファ数 8情報分 */
            unRet = Hedio864.ed864_EtnSearch(byInf, ref usMax, 2000);     /* 2000ms waitting time/max 4 informations) */
            if ((unRet != 0) && (unRet != Hedio864.NETDEV_NOTFOUND)) return unRet;
            if (usPort != 0)
            {
                usTprt = usPort;
            }
            // ﾎﾟｰﾄは同じはずなので先頭情報を使用
            else if (unRet == 0)
            {
                usTprt = (ushort)((ushort)byInf[7] * 0x100 + (ushort)byInf[6]);
            }
            else
            {
                usTprt = 10010;
            }
            byAdr = System.Text.Encoding.ASCII.GetBytes("127.0.0.1");
            unRet = Hedio864.ed864_SetProgLink(byAdr, usTprt, RESP_TMO);    /* Local loopback address */
            if (unRet != 0) return unRet;
        }
        else
        {
            return Hedio864.ILLEGAL_NETPARAM;
        }
        return unRet;
    }

    /// <summary>
    /// 常駐プログラムソフトウェアバージョン読出
    /// </summary>
    /// <param name="usPver">常駐ﾌﾟﾛｸﾞﾗﾑﾊﾞｰｼﾞｮﾝ</param>
    /// <param name="usLver">ｱｸｾｽﾗｲﾌﾞﾗﾘﾊﾞｰｼﾞｮﾝ</param>
    /// <returns>0:正常，0以外:異常</returns>
    public static uint hed864_GetProgVersion(ref ushort usPver, ref ushort usLver)
    {
        uint unRet = 0;

        // プログラムバージョン読出
        unRet = Hedio864.ed864_GetProgVersion(ref usPver, ref usLver);
        return unRet;
    }

    //=============================================================================
    //							動作情報取得
    //=============================================================================
    /// <summary>
    /// コマンド実行エラーの取得
    /// <param name="sTyp">デバイスタイプ</param>
    /// <param name="sBid">ボードID</param>
    /// <param name="err">エラー情報格納アドレス
    /// <returns>0:正常，0以外:異常</returns>
    public static uint hed864_GetCmdError(short sTyp, short sBid, ref uint err)
    {
        byte[] by0xE1Cmd = new byte[10] {
            0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
			0xE1, 0x00
        };
        byte[] by0xE0Cmd = new byte[10] {
            0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
            0xE0, 0x01
        };

        byte[] byRes = new byte[24];
        ushort wCsz;
        ushort wRsz;
        uint dwRet;			// 戻り値

        by0xE1Cmd[0] = (byte)sBid;	// set BID
        wCsz = (ushort)by0xE1Cmd.Length;	// set command data size
        wRsz = (ushort)byRes.Length;		// set response data size
        dwRet = Hedio864.ed864_EtnSend(sTyp, sBid, by0xE1Cmd, ref wCsz);
        if (dwRet != 0) return dwRet;
        dwRet = Hedio864.ed864_EtnRecv(sTyp, sBid, byRes, ref wRsz);
        if (dwRet != 0) return dwRet;
        err = (uint)byRes[8] + ((uint)byRes[9] * 0x10) + ((uint)byRes[10] * 0x100) + ((uint)byRes[11] * 0x1000);    // エラー情報
        if (err != 0)
        {		// エラーがある場合はクリアコマンド実行
            by0xE0Cmd[0] = (byte)sBid;	        // set BID
            wCsz = (ushort)by0xE0Cmd.Length;	// set command data size
            wRsz = 8;		        			// set response data size(header only)
            dwRet = Hedio864.ed864_EtnSend(sTyp, sBid, by0xE0Cmd, ref wCsz);
            if (dwRet != 0) return dwRet;
            dwRet = Hedio864.ed864_EtnRecv(sTyp, sBid, byRes, ref wRsz);
        }
        return dwRet;
    }
}